#include <strstrea.h>
#include <fstream.h>
#include <ctype.h>

int WIDTH[3] = {8,10,6};

char keywords1[31][8] = {".186",".286",".286C",".286P",".287",".386",".386C",".386P",
                         ".387",".486",".486C",".486P",".487",".586",".586C",".586P",
                         ".587",".80086",".8087",".MODEL",".RADIX",".STACK","MODEL",
                         "RADIX","STACK","IDEAL","JUMPS","LOCALS","ENDS","ENDP","END"};

char keywords2[3][8] = {"SEGMENT","PROC","STRUCT"};
char keywords3[6][8] = {".CODE",".DATA",".FARDATA","CODESEG","DATASEG","FARDATA"};

void PrintHelp()
{
  cout << "Assembler source file converter. Copyright (c) 1999 MATI Inc.\n";
  cout << "This converter allows to improve readability of your assembler source file.\n";
  cout << "For example, if you have the following code:\n";
  cout << "\t\"n1:mov data[bp], bl\"\n";
  cout << "\t\"  mov bl,0         \"\n";
  cout << "\t\"test e,2           \"\n";
  cout << "\t\"jz n2              \"\n";
  cout << "\t\"mov bl, al         \"\n";
  cout << "\nASMCONV could convert it in something like this:\n";
  cout << "\t\"N1:     MOV       DATA[BP], BL\"\n";
  cout << "\t\"        MOV       BL,   0     \"\n";
  cout << "\t\"        TEST      E,    2     \"\n";
  cout << "\t\"        JZ        N2          \"\n";
  cout << "\t\"        MOV       BL,   AL    \"\n";
  cout << "\n\tUsage: ASMCONV.EXE <in_file> <out_file>\n\n";
  cout << "Where  \"in_file\"  is an assembler source file, and \"out_file\" is converted\n";
  cout << "assembler source file, which ASMCONV will generate for you.\n";
  cout << "Warning: \"in_file\" must not be the same as \"out_file\"\n";
}

unsigned main(int argc, char **argv)
{
  char *buffer, *temp = new char[200];
  int i, j, flag, flag2;

  if (argc < 3)
  {
    PrintHelp();
    return 0;
  }

  ifstream in(argv[1]);
  if (!in)
  {
    cout << "File not found\n";
    return 1;
  }

  ofstream out(argv[2]);
  if (!out)
  {
    cout << "Cannot create file\n";
    return 1;
  }

  while (in)
  {
    in.getline(temp, 200, '\n');

    buffer = temp;
    if (!*buffer) continue;

    for (i = 0; i < strlen(buffer); i++) buffer[i] = toupper(buffer[i]);
    while (*buffer == ' ') buffer++;

    if (*buffer == ';')
    {
      out << buffer << '\n';
      continue;
    }

    i = j = 0;
    while ((buffer[i] != ' ') && (buffer[i] != '\t') && (buffer[i] != '\n'))
    {
      i++;
      if (buffer[i] == ':') j++;
    }

    if (j)
    {
      i = 0;
      while (*buffer != ':')
      {
        out << *buffer;
        buffer++;
        i++;
      }

      out << ':';
      buffer++;

      if (i > WIDTH[0] - 1)
        out << '\n';
      else
      {
        flag = 1;
        for(i++; i < WIDTH[0]; i++) out << ' ';
      }
    }

    while ((*buffer == ' ') || (*buffer == '\t') || (*buffer == '\n'))
      buffer++;

    if (*buffer)
    {
      j = 0;
      flag2 = 0;

      for (i = 0; i < 31; i++)
        if (strstr(buffer, keywords1[i]))
        {
          flag2 = 1;
          j = 1;
          break;
        }

      if (!j)
      for (i = 0; i < 3; i++)
        if (strstr(buffer, keywords2[i]))
        {
          out << '\n';
          flag2 = 1;
          break;
        }

     if (!j)
     for (i = 0; i < 6; i++)
        if (strstr(buffer, keywords3[i]))
        {
          out << '\n';
          flag2 = 1;
          j = 1;
          break;
        }

      if (!flag && !flag2)
        for (i = 0; i < WIDTH[0]; i++)
          out << ' ';
      flag = 0;
      if (!flag2) flag++;

      while (*buffer)
      {
        i = 0;
        while (*buffer && (*buffer != ' ') && (*buffer != '\t') && (*buffer != '\n') && (*buffer != ','))
        {
          out << *buffer;
          buffer++;
          i++;
        }

        if (*buffer == ',')
        {
          out << *buffer;
          buffer++;
          i++;
        }

        if (flag < 3)
        {
          if (i > WIDTH[flag] - 2)
            out << ' ';
          else
            for(i++; i < WIDTH[flag] + 1; i++) out << ' ';
        }
        else out << ' ';

/*        switch (flag)
        {
          case 0: if (i > 6)
                    out << ' ';
                  else
                    for(i++; i < 9; i++) out << ' ';
                  break;
          case 1: if (i > 8)
                    out << ' ';
                  else
                    for(i++; i < 11; i++) out << ' ';
                  break;
          case 2: if (i > 4)
                    out << ' ';
                  else
                    for(i++; i < 7; i++) out << ' ';
                  break;
          default: out << ' ';
        } */

        while ((*buffer == ' ') || (*buffer == '\t') || (*buffer == '\n'))
          buffer++;

        flag++;
      }

      out << '\n';
      if (j) out << '\n';
      flag = 0;
    }
  }

  in.close();
  out.close();
  delete temp;

  return 0;
}
