#include <graphics.h>
#include <fstream.h>
#include <string.h>
#include <stdlib.h>
#include <conio.h>
#include <time.h>
#include <new.h>
#include "objects.cpp"
#include "mouse.cpp"
#include "procs.h"
#include "error.h"
#include "defs.h"

#define VIDEOMODE           2

class TEditor {
    TObject *beg, *end;
    char filename[40];
    TEditBox *EditBox;
    int x, y, b, butt_flag;
    int texture;
    bool pressed;
    void DrawDesktop();
    void AddControl(TObject *);
    void Handle(TEvent& event);
    void ClearEvent(TEvent& event);
    void InitControls();
    void New();
    int Save();
    int Load();
    int CheckModify();
  public:
    TEditor();
    int LoadLevel(char *filename);
    void Run();
    ~TEditor();
};

TEditor::TEditor()
{
  beg = end = NULL;
  pressed = False;
  butt_flag = False;
  int gd = installuserdriver("BGI256V2", NULL);
  int gm = VIDEOMODE;
  initgraph(&gd, &gm, "");
  int res = graphresult();
  if ((res != grOk) && (res != 781))
  {
    graph_error(res);
    exit(1);
  }
  for (int i = 1; i < 256; i++)
    setrgbpalette(i, *(COLORMAP_ADDR + i * 4 + 2) / 4,
                     *(COLORMAP_ADDR + i * 4 + 1) / 4,
                     *(COLORMAP_ADDR + i * 4) / 4);
  DrawDesktop();
  InitControls();

  mouse_init();
  set_mouse_range(0, 623, 0, 460);
  mouse_set_pos(getmaxx() / 2, getmaxy() / 2 + 10);
  mouse_show();

  New();
}

int TEditor::CheckModify()
{
  if (!modified) return 0;

  if (MessageBox("Level modified. Save?", MB_YESNO) == ID_YES)
    return Save();

  return 0;
}

void TEditor::New()
{
  int i, j;

  if (CheckModify()) return;

  mouse_hide();

  EditBox -> DrawSelected();
  numobjects = 0;
  modified = False;
  texture = 0;
  for (i = 0; i < 16; i++)
    for (j = 0; j < 28; j++)
      Level[i][j] = 255;
  EditBox -> Draw();
  strcpy(filename, "noname.lev");

  mouse_show();
}

int TEditor::Save()
{
  char text[40];
  strcpy(text, filename);
  InputBox("Enter file name to save level to:", text);

  ofstream out(text, ios::binary);

  if (!out)
  {
    MessageBox("Error: cannot create file", MB_OK);
    return 1;
  }
  strcpy(filename, text);

  out.write((char *)Level, 448);
  out.write((char *)Objects, sizeof(Objects));
  out.write((char *)&numobjects, 2);
  out.write((char *)&texture, 2);
  out.close();

  modified = False;
  return 0;
}

int TEditor::Load()
{
  if (CheckModify()) return 0;

  char text[40];
  strcpy(text, filename);
  InputBox("Enter file name to load level from:", text);

  return LoadLevel(text);
}

int TEditor::LoadLevel(char *filename)
{
  ifstream in(filename, ios::binary);

  if (!in)
  {
    MessageBox("Error: cannot open file", MB_OK);
    return 1;
  }
  strcpy(TEditor::filename, filename);

  in.read((char *)Level, 448);
  in.read((char *)Objects, sizeof(Objects));
  in.read((char *)&numobjects, 2);
  in.read((char *)&texture, 2);
  in.close();

  mouse_hide();
  EditBox -> Draw();
  EditBox -> DrawObjects();
  mouse_show();

  modified = False;
  return 0;
}

void TEditor::InitControls()
{
  int i, j;

  for (i = 0; i < 22; i++)
    for (j = 0; j < 5; j++)
      if (j * 22 + i < 107)
        AddControl(new TBitBtn(495 + j * 27, 10 + i * 21, 27, 21, 1000 + j * 22 + i, j * 22 + i));

  AddControl(new TTextButton(10, 450, 50, 20, CM_NEW, "New"));
  AddControl(new TTextButton(70, 450, 50, 20, CM_LOAD, "Load"));
  AddControl(new TTextButton(130, 450, 50, 20, CM_SAVE, "Save"));
  AddControl(new TTextButton(190, 450, 50, 20, CM_CLEAR, "Clear"));
  AddControl(new TTextButton(410, 450, 50, 20, CM_EXIT, "Quit"));

  AddControl(new TBitBtn(385, 10, 27, 21, 2004, (unsigned far *)DINO_ADDR));
  AddControl(new TBitBtn(415, 10, 27, 21, 2005, (unsigned far *)FISH_ADDR));
  AddControl(new TBitBtn(445, 10, 27, 21, 2006, (unsigned far *)BONUS_ADDR));
  AddControl(new TBitBtn(385, 35, 27, 21, 2007, (unsigned far *)CAPS_ADDR));
  AddControl(new TBitBtn(415, 35, 27, 21, 2008, (unsigned far *)SECRET_ADDR));
  AddControl(new TBitBtn(445, 35, 27, 21, 2009, (unsigned far *)MONEY_ADDR));
  AddControl(new TBitBtn(385, 60, 27, 21, 2010, (unsigned far *)BALL_ADDR));
  AddControl(new TBitBtn(415, 60, 27, 21, 2011, (unsigned far *)PLATFORM_ADDR));
  AddControl(new TBitBtn(445, 60, 27, 21, 2012, (unsigned far *)FIREBALL_ADDR));
  AddControl(new TBitBtn(385, 85, 27, 35, 2001, (unsigned far *)MARIOS_ADDR));
  AddControl(new TBitBtn(415, 85, 27, 31, 2002, (unsigned far *)TURTLES_ADDR));
  AddControl(new TBitBtn(445, 85, 27, 21, 2003, (unsigned far *)DOTS_ADDR));
  AddControl(new TBitBtn(385, 125, 33, 27, 2013, (unsigned far *)FLOWERS_ADDR));

  AddControl(new TTextButton(425, 125, 50, 27, 2014, "Del."));
  AddControl(new TTextButton(385, 160, 90, 20, 2015, "hurt"));
  AddControl(new TTextButton(385, 190, 90, 20, 2016, "teleport"));
  AddControl(new TTextButton(385, 220, 90, 20, 2017, "level"));
  AddControl(new TTextButton(385, 260, 90, 20, CM_TEX, "Texture..."));
  AddControl(new TTextButton(385, 290, 90, 20, CM_PROP, "Property"));

  EditBox = new TEditBox(PANEL_X + 2, PANEL_Y + 2);
}

TEditor::~TEditor()
{
  TObject *p = beg;

  while (p != NULL)
  {
    beg = p -> next;
    delete p;
    p = beg;
  }

  closegraph();
  restorecrtmode();
}

void TEditor::AddControl(TObject *control)
{
  if (beg == NULL)
    beg = control;
  else
    end -> next = control;
  end = control;
}

void TEditor::DrawDesktop()
{
  Draw3DBox(485, 0, getmaxx() - 485, getmaxy());
  Draw3DBox(375, 0, 110, 250);
  Draw3DBox(375, 250, 110, 190);
  Draw3DBox(0, 440, getmaxx() - 154, getmaxy() - 440);
  Draw3DBox(PANEL_X - 4, PANEL_Y - 4, 332, 404);
  Draw3DHole(PANEL_X, PANEL_Y, 324, 396, BLACK);
  Draw3DHole(250, 451, 24, 18, BLACK);
}

void my_handler()
{
  cout << "Error: not enough memory\n";
  exit(1);
}

void TEditor::Handle(TEvent& event)
{
  int xt, yt, bt;
  unsigned char k;
  char tex[40];
  TObject *p;

  asm
  {
    in     al,  60h
    mov    k,   al
  }

  if (kbhit())
  {
    event.key = getch();
    if (!event.key) event.scan = getch();
  }

  mouse_read(xt, yt, bt);
  event.x = xt;
  event.y = yt;
  event.b = bt;

  if (k == 57)
    butt_flag = True;
  if (k == 185)
    butt_flag = False;

  if (butt_flag) event.b = 1;

  switch (event.scan)
  {
    case 77: mouse_hide(); mouse_set_pos(xt + 5, yt); mouse_show(); break;
    case 75: mouse_hide(); mouse_set_pos(xt - 5, yt); mouse_show(); break;
    case 72: mouse_hide(); mouse_set_pos(xt, yt - 5); mouse_show(); break;
    case 80: mouse_hide(); mouse_set_pos(xt, yt + 5); mouse_show(); break;
  }

  p = beg;
  while (p != NULL)
  {
    p -> HandleEvent(event);
    p = p -> next;
  }
  EditBox -> HandleEvent(event);

  if ((event.command > 999) && (event.command < 1110))
  {
    EditBox -> selected = event.command - 1000;
    EditBox -> DrawSelected();
  }

  switch (event.command)
  {
    case CM_CLEAR: EditBox -> selected = 255;
                   EditBox -> DrawSelected();
                   break;
    case CM_LOAD: Load(); break;
    case CM_SAVE: Save(); break;
    case CM_EXIT: CheckModify(); break;
    case CM_NEW: New(); break;
    case CM_TEX: itoa(texture, tex, 10);
                 InputBox("Enter new texture number:", tex);
                 texture = atoi(tex);
                 break;
  }

  if (event.scan == 45) event.command = CM_EXIT;
}

void TEditor::ClearEvent(TEvent& event)
{
  if (!(((event.command > 2000) && (event.command < 2018)) || (event.command == CM_PROP)))
    event.command = 0;
  event.key = 0;
  event.scan = 0;
  event.x = 0;
  event.y = 0;
  event.b = 0;
}

void TEditor::Run()
{
  TEvent event;

  do
  {
    ClearEvent(event);
    Handle(event);
    DrawMouse();
  } while (event.command != CM_EXIT);
}

void main(int argc, char **argv)
{
  set_new_handler(my_handler);

  TEditor Editor;
  if (argc == 2)
    if (!Editor.LoadLevel(argv[1]))
      MessageBox("Level loaded successfully!", MB_OK);
  else
    MessageBox("Mario Level Editor. (c) 1999 MATI Inc.", MB_OK);
  Editor.Run();
}
