#include <iostream.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <mem.h>
#include <dos.h>
#include "tmario.cpp"
#include "graphics.h"
#include "texture.h"
#include "procs.h"
#include "defs.h"

struct LVL_PARAM {
  char LevelNumber;
  char DestLevel;
  char Texture;
  char NeedCoins;
  char temp;
  char Name[20];
};

void interrupt (*OldTimer)(...);
void interrupt (*OldKeyboard)(...);
unsigned TreeFlag = 0, keycode, allow_scroll;

class TGame {
    unsigned cur_level;
    unsigned cur_page;
    int screen_y;
    LVL_PARAM LevelParams;
    TMario *mario;
    void Redraw(unsigned flag);
    void HandleTimer();
    void friend interrupt KeyboardHandler(...);
    void friend interrupt TimerHandler(...);
  public:
    TGame();
    void FlipPage();
    void Intro();
    void Run();
    void LevelScreen(unsigned level);
    void StartLevel(unsigned level);
    ~TGame();
} *Game;

void interrupt TimerHandler(...)
{
  static int count, c1 = 0, c2 = 0, c3 = 0;

  if (allow_scroll)
  {
    if (!c1)
    {
      c1 = 3;
      SCROLL_PAL(12, 2);
    }
    if (!c2)
    {
      c2 = 6;
      SCROLL_PAL(158, 1);
    }
    if (!c3)
    {
      c3 = 1;
      SCROLL_PAL(7, 4);
    }
  }

  if (c1) c1--;
  if (c2) c2--;
  if (c3) c3--;

  count++;
  count %= 6;
  if (!count) TreeFlag = 1;

  OldTimer();
}

void interrupt KeyboardHandler(...)
{
  keycode = inportb(0x60);
  OldKeyboard();
}

TGame::TGame()
{
  allow_scroll = 0;
  OldTimer = _dos_getvect(0x1C);
  OldKeyboard = _dos_getvect(0x09);
  asm cli;
  _dos_setvect(0x1C, TimerHandler);
  _dos_setvect(0x09, KeyboardHandler);
  asm sti;
  SET_13H_MODE();
  SET_NULL_PAL();
  cur_level = START_LEVEL;
  screen_y = 0;
  mario = NULL;
  PAGE = PAGE_0;
  SET_PAGE(PAGE ^ PAGE_1);
  Game = this;
}

void TGame::FlipPage()
{
  PAGE ^= PAGE_1;
  SET_PAGE(PAGE ^ PAGE_1 + screen_y * 80);
}

void TGame::Intro()
{
  int i, j;

  for (i = 0; i < 50; i++)
    SET_TABLE_PAL(i + 200, 0, i * 3, i * 5);

  for (i = 0; i < 200; i++)
    H_LINE(0, i, 320, 249 - i / 4);

  for (i = 0; i < INTRO_OFFS; i++)
  {
    int x = *(INTRO_ADDR + i * 3 + 1);
    int y = *(INTRO_ADDR + i * 3 + 2);
    int offs = SPR_OFS[*(INTRO_ADDR + i * 3)];
    PUT_RENDER_IMAGE(x * 20, y * 14 + 20, 20, 14, SPRITES_ADDR + offs);
  }
  PUT_RENDER_IMAGE(79, 20, 160, 50, INTRO_ADDR + INTRO_OFFS);

  FlipPage();
  FADE_IN(COLORMAP_ADDR);
  ClearKbd();
  Delay(2000);

  do
  {
    for (j = 0; j < 80; j++)
    {
      if (!j)
        for (i = 0; i < 50; i++)
          SET_TABLE_PAL(i + 200, i * 5, i * 3, 0);
      if (j == 20)
        for (i = 0; i < 50; i++)
          SET_TABLE_PAL(i + 200, i * 3, i * 5, 0);
      if (j == 40)
        for (i = 0; i < 50; i++)
          SET_TABLE_PAL(i + 200, i * 5, i * 3, i * 5);
      if (j == 60)
        for (i = 0; i < 50; i++)
          SET_TABLE_PAL(i + 200, i * 2, i * 5, i * 2);

      if (!(j % 20))
      {
        VERT_RET();
        SET_ALL_PAL(COLORMAP_ADDR);
      }

      if (kbhit()) break;
      Delay(100);
    }
  } while (!kbhit());
}

void TGame::HandleTimer()
{
  int i, j, sprite, k = screen_y / 14 + 16;
  if (k > 28) k = 28;

  for (i = 0; i < 16; i++)
    for (j = 0; j < k; j++)
    {
      sprite = LevelTable[i][j];
      if ((sprite >= TREE1_START) && (sprite < TREE2_START))
      {
        LevelTable[i][j] = sprite + TREE_SIZE;
        DrawTable[i][j] = 1;
        DrawTable[i + 16][j] = 1;
      }
      else if ((sprite >= TREE2_START) && (sprite < TREE2_START + TREE_SIZE))
      {
        LevelTable[i][j] = sprite - TREE_SIZE;
        DrawTable[i][j] = 1;
        DrawTable[i + 16][j] = 1;
      }
    }
  TreeFlag = 0;
}

void TGame::Redraw(unsigned flag)
{
  int i, j, sprite;

  for (i = 0; i < 16; i++)
    for (j = 0; j < 28; j++)
      if (DrawTable[i + flag * 16][j])
      {
        sprite = LevelTable[i][j];
        PutTexture(i * 20, j * 14, LevelParams.Texture);
        PUT_RENDER_IMAGE(i * 20, j * 14, 20, 14, SPRITES_ADDR + SPR_OFS[sprite]);
      }
}

void TGame::Run()
{
  unsigned loc_page = 0;
  LevelScreen(cur_level);
  StartLevel(cur_level);

  do
  {
    VERT_RET();
    if (TreeFlag && !loc_page) HandleTimer();
    mario -> HandleEvent(keycode);
    Redraw(loc_page);
    mario -> Draw();
    screen_y = mario -> GetY() - 100;
    if (screen_y < 0) screen_y = 0;
    if (screen_y > 192) screen_y = 192;
    FlipPage();
    allow_scroll = 1;
    memset(DrawTable, 0, LEVEL_DATA * (loc_page + 1));
    loc_page ^= 1;
    ClearKbd();
  } while (keycode != 1);
}

void TGame::LevelScreen(unsigned level)
{
  int i, j, k;
  char str[30];

  allow_scroll = 0;

  for (k = 0; k < 20; k++)
  {
    SET_TABLE_PAL(k + LS_START_COLOR, (k + 1) * 10, (k + 1) * 10, 0);
    SET_TABLE_PAL(k + LS_START_COLOR + 20, 0, (k + 1) * 6, 0);
  }
  SET_ALL_PAL(COLORMAP_ADDR);

  for (i = 0; i < 10; i++)
    for (j = 0; j < 8; j++)
      for (k = 0; k < 10; k++)
      {
        Rect1(j * 40 + k + (i % 2) * 20, i * 20 + k + (cur_page ^ 1) * PAGE_1, 20 - k * 2, 20 - k * 2,
              LS_START_COLOR + k);
        Rect1(j * 40 + k + (1 - (i % 2)) * 20, i * 20 + k + (cur_page ^ 1) * PAGE_1, 20 - k * 2, 20 - k * 2,
              LS_START_COLOR + k + 20);
      }

  NumToStr(level, str);
  PutShadowText1(160 - strlen("World") * 18, 40, "World", 14, 4, 4, 5);
  PutShadowText1(160 - strlen(str) * 13, 120, str, 14, 3, 3, 140);

  FlipPage();
  Delay(3000);
  ClearKbd();
}

void TGame::StartLevel(unsigned level)
{
  memset(DrawTable, 1, LEVEL_DATA * 2);
  memcpy(LevelTable, LEVELS_ADDR + level * LEVEL_SIZE, LEVEL_DATA);
  memcpy(&LevelParams, LEVELS_ADDR + level * LEVEL_SIZE + LEVEL_DATA, sizeof(LevelParams));
  ClearKbd();

  if (!LevelParams.Texture)
  {
    for (int i = 0; i < 32; i++)
      SET_TABLE_PAL(i + LS_START_COLOR + 40, 0, i * 4, i * 8);
    SET_ALL_PAL(COLORMAP_ADDR);
  }

  if (mario != NULL) delete mario;
  mario = new TMario(60, 28);
}

TGame::~TGame()
{
  asm cli;
  _dos_setvect(0x09, OldKeyboard);
  _dos_setvect(0x1C, OldTimer);
  asm sti;
  SET_03H_MODE();
  cout << "MARIO C++ Version. Copyright (c) 1999 by MATI Inc.\n";
}

