#ifndef _GRAPHICS_H_
#define _GRAPHICS_H_
#include <dos.h>

// functions, implemented in module MODE.ASM

extern "C" void far SET_13H_MODE();
extern "C" void far SET_03H_MODE();
extern "C" void far SET_PAGE(unsigned page_addr);
extern "C" void far VERT_RET();

// functions, implemented in module PALETTE.ASM

extern "C" void far SET_RGB(unsigned n, unsigned r, unsigned g, unsigned b);
extern "C" void far SET_ALL_PAL(unsigned char far *palette);
extern "C" void far SET_NULL_PAL();
extern "C" void far FADE_IN(unsigned char far *palette);
extern "C" void far FADE_OUT(unsigned char far *palette);
extern "C" void far SCROLL_PAL(unsigned start, unsigned count);

// functions, implemented in module GRAPHICS.ASM

extern "C" void far PUT_PIXEL(unsigned x, unsigned y, unsigned color);
extern "C" void far PUT_IMAGE(unsigned x, unsigned y,
                              unsigned w, unsigned h, unsigned char far *data);
extern "C" void far PUT_RENDER_IMAGE(unsigned x, unsigned y,
                                     unsigned w, unsigned h, unsigned char far *data);
extern "C" void far H_LINE(unsigned x, unsigned y, unsigned w, unsigned color);
extern "C" void far V_LINE(unsigned x, unsigned y, unsigned h, unsigned color);
extern "C" void far DIAGONAL_LINE1(unsigned x, unsigned y, unsigned h, unsigned color);
extern "C" void far DIAGONAL_LINE2(unsigned x, unsigned y, unsigned h, unsigned color);
extern "C" void far BAR(unsigned x, unsigned y, unsigned w, unsigned h, unsigned color);
extern "C" void far FLIP(unsigned char far *data, unsigned w, unsigned h);

// data structures, implemented in data module MAINDATA.ASM

extern unsigned far SPRITES;
extern unsigned far LEVELS;
extern unsigned far INTRO;
extern unsigned far FONTS;
extern unsigned far COLORMAP;
extern unsigned far PAGE;

extern unsigned char far *SPRITES_ADDR = (unsigned char far *)(&SPRITES + 1);
extern unsigned char far *INTRO_ADDR = (unsigned char far *)(&INTRO + 1);
extern unsigned char far *FONTS_ADDR = (unsigned char far *)(&FONTS + 1);
extern unsigned char far *COLORMAP_ADDR = (unsigned char far *)(&COLORMAP + 1);
extern unsigned char far *LEVELS_ADDR = (unsigned char far *)(&LEVELS + 1);

// self-implemented functions

void SET_TABLE_PAL(unsigned n, unsigned r, unsigned g, unsigned b);
void Rect1(unsigned x, unsigned y, unsigned w, unsigned h, unsigned color);
void Rect2(unsigned x1, unsigned y1, unsigned x2, unsigned y2, unsigned color);

void PutText(unsigned x, unsigned y, char *text, unsigned font, unsigned size_x, unsigned size_y, unsigned color);
void PutChar(unsigned x, unsigned y, char c, unsigned font, unsigned size_x, unsigned size_y, unsigned color);
void PutShadowText(unsigned x, unsigned y, char *text, unsigned font, unsigned size_x, unsigned size_y, unsigned color);
void PutShadowChar(unsigned x, unsigned y, char c, unsigned font, unsigned size_x, unsigned size_y, unsigned color);

void PutText1(unsigned x, unsigned y, char *text, unsigned font, unsigned size_x, unsigned size_y, unsigned color);
void PutChar1(unsigned x, unsigned y, char c, unsigned font, unsigned size_x, unsigned size_y, unsigned color);
void PutShadowText1(unsigned x, unsigned y, char *text, unsigned font, unsigned size_x, unsigned size_y, unsigned color);
void PutShadowChar1(unsigned x, unsigned y, char c, unsigned font, unsigned size_x, unsigned size_y, unsigned color);

void SET_TABLE_PAL(unsigned n, unsigned r, unsigned g, unsigned b)
{
  *(COLORMAP_ADDR + n * 4 + 2) = r;
  *(COLORMAP_ADDR + n * 4 + 1) = g;
  *(COLORMAP_ADDR + n * 4) = b;
}

void Rect1(unsigned x, unsigned y, unsigned w, unsigned h, unsigned color)
{
  H_LINE(x, y, w, color);
  H_LINE(x, y + h - 1, w, color);
  V_LINE(x, y, h, color);
  V_LINE(x + w - 1, y, h, color);
}

void Rect2(unsigned x1, unsigned y1, unsigned x2, unsigned y2, unsigned color)
{
  if (x1 > x2) { x1 ^= x2; x2 = x1 ^ x2; x1 ^= x1; }
  if (y1 > y2) { y1 ^= y2; y2 = y1 ^ y2; y1 ^= y1; }

  H_LINE(x1, y1, x2 - x1 + 1, color);
  H_LINE(x1, y2, x2 - x1 + 1, color);
  V_LINE(x1, y1, y2 - y1 + 1, color);
  V_LINE(x2, y1, y2 - y1 + 1, color);
}

void PutText(unsigned x, unsigned y, char *text, unsigned font, unsigned size_x, unsigned size_y, unsigned color)
{
  int i;

  for (i = 0; i < strlen(text); i++)
    PutChar(x + i * size_x * 9, y, text[i], font, size_x, size_y, color);
}

void PutChar(unsigned x, unsigned y, char c, unsigned font, unsigned size_x, unsigned size_y, unsigned color)
{
  unsigned offset;
  int i, j, xx, yy = y;
  char b;

  switch (font)
  {
    case 14: offset = 2048; break;
    case 16: offset = 5632; break;
    default: offset = 0;
  }

  offset += (unsigned)c * font;

  for (i = 0; i < font; i++, yy += size_y)
  {
    b = *(FONTS_ADDR + offset + i);
    xx = x;
    for (j = 0; j < 8; j++, xx += size_x)
      if ((b << j) & 0x80)
        BAR(xx, yy, size_x, size_y, color);
  }
}

void PutShadowText(unsigned x, unsigned y, char *text, unsigned font, unsigned size_x, unsigned size_y, unsigned color)
{
  int i;

  for (i = 0; i < strlen(text); i++)
    PutShadowChar(x + i * size_x * 9, y, text[i], font, size_x, size_y, color);
}

void PutShadowChar(unsigned x, unsigned y, char c, unsigned font, unsigned size_x, unsigned size_y, unsigned color)
{
  unsigned offset;
  int i, j, xx, yy = y;
  char b;

  switch (font)
  {
    case 14: offset = 2048; break;
    case 16: offset = 5632; break;
    default: offset = 0;
  }

  offset += (unsigned)c * font;

  for (i = 0; i < font; i++, yy += size_y)
  {
    b = *(FONTS_ADDR + offset + i);
    xx = x;
    for (j = 0; j < 8; j++, xx += size_x)
      if ((b << j) & 0x80)
      {
        BAR(xx + 1, yy + 1, size_x, size_y, 0);
        BAR(xx, yy, size_x, size_y, color);
      }
  }
}

void PutText1(unsigned x, unsigned y, char *text, unsigned font, unsigned size_x, unsigned size_y, unsigned color)
{
  int i;

  for (i = 0; i < strlen(text); i++)
    PutChar1(x + i * size_x * 9, y, text[i], font, size_x, size_y, color);
}

void PutChar1(unsigned x, unsigned y, char c, unsigned font, unsigned size_x, unsigned size_y, unsigned color)
{
  unsigned offset;
  int i, j, xx, yy = y;
  char b;

  switch (font)
  {
    case 14: offset = 2048; break;
    case 16: offset = 5632; break;
    default: offset = 0;
  }

  offset += (unsigned)c * font;

  for (i = 0; i < font; i++, yy += size_y)
  {
    b = *(FONTS_ADDR + offset + i);
    xx = x;
    for (j = 0; j < 8; j++, xx += size_x)
      if ((b << j) & 0x80)
        Rect1(xx, yy, size_x, size_y, color);
  }
}

void PutShadowText1(unsigned x, unsigned y, char *text, unsigned font, unsigned size_x, unsigned size_y, unsigned color)
{
  int i;

  for (i = 0; i < strlen(text); i++)
    PutShadowChar1(x + i * size_x * 9, y, text[i], font, size_x, size_y, color);
}

void PutShadowChar1(unsigned x, unsigned y, char c, unsigned font, unsigned size_x, unsigned size_y, unsigned color)
{
  unsigned offset;
  int i, j, xx, yy = y;
  char b;

  switch (font)
  {
    case 14: offset = 2048; break;
    case 16: offset = 5632; break;
    default: offset = 0;
  }

  offset += (unsigned)c * font;

  for (i = 0; i < font; i++, yy += size_y)
  {
    b = *(FONTS_ADDR + offset + i);
    xx = x;
    for (j = 0; j < 8; j++, xx += size_x)
      if ((b << j) & 0x80)
      {
        Rect1(xx + 1, yy + 1, size_x, size_y, 0);
        Rect1(xx, yy, size_x, size_y, color);
      }
  }
}


#endif
